-- Creates a scheduler class, for "cron" and "at"-like functionality
-- Going to handle "at" first.  But the basic idea holds

YAK_SCHEDULER = {}

function YAK_SCHEDULER:Create()
   result = {at_queue = {}, cron_queue = {}, log = YAK_LOGGER:Create()}
   setmetatable(result, self)
   self.__index = self
   return result
end

function YAK_SCHEDULER:OnUpdate()
   local now = GetTime()
   self:UpdateAtQueue(now)
   self:UpdateCronQueue(now)
end

function YAK_SCHEDULER:UpdateAtQueue( now )
   -- the "at queue" is a table holding {at_time, id, closure} tuples indexed
   -- by integer
   local tasks_to_do = {}
   for k,v in ipairs(self.at_queue) do
      if now > v[1] then
	 table.insert(tasks_to_do,k)
      end
   end
   for _,id in tasks_to_do do
      local v = self.at_queue[id]
      --do the task
      self.log:Log("Doing at-job " .. id .. " with id ".. v[2])
      v[3]()
      --then remove it from the at queue
      table.remove(self.at_queue, id)
   end
end

function YAK_SCHEDULER:UpdateCronQueue( now )
   for k,v in self.cron_queue do
      if v[2] < now then
	 self.log:Log("Doing cron-job " .. k )
	 v[3]()
	 v[2] = now + v[1]
      end
   end
end

function YAK_SCHEDULER:At(time, id, closure)
   table.insert( self.at_queue, {time, id, closure} )
end

function YAK_SCHEDULER:In( dt, id, closure )
   table.insert( self.at_queue, {GetTime()+dt, id, closure} )
end

function YAK_SCHEDULER:Every( dt, id, closure )
   self.cron_queue[id] = {dt, GetTime()+dt, closure}
end

function YAK_SCHEDULER:SetCronDelay(id, new_dt)
   self.cron_queue[id][1] = new_dt
end

function YAK_SCHEDULER:RemoveCronJob(id)
   table.remove(self.cron_queue, id)
end